@@ -63,7 +63,7 @@ typedef struct controller_info {
     int            tcpfd;		/* When connected, the file
                                            descriptor for the TCP
                                            port used for I/O. */
-    struct sockaddr_in remote;		/* The socket address of who
+    struct sockaddr_storage remote;	/* The socket address of who
 					   is connected to this port. */
 
     unsigned char inbuf[INBUF_SIZE+1];	/* Buffer to receive command on. */
@@ -228,7 +228,7 @@ controller_output(struct controller_info *cntlr,
 void
 controller_write(struct controller_info *cntlr, char *data, int count)
 {
-    write(cntlr->tcpfd, data, count);
+    write_ignore_fail(cntlr->tcpfd, data, count);
 }
 
 static void
@@ -543,37 +543,22 @@ handle_tcp_fd_write(int fd, void *data)
     telnet_data_t *td = &cntlr->tn_data;
     int write_count;
 
-    if (td->out_telnet_cmd_size > 0) {
-	write_count = write(cntlr->tcpfd,
-			    &(td->out_telnet_cmd[0]),
-			    td->out_telnet_cmd_size);
-	if (write_count == -1) {
-	    if (errno == EINTR) {
-		/* EINTR means we were interrupted, just retry by returning. */
-		goto out;
-	    }
+    if (buffer_cursize(&td->out_telnet_cmd) > 0) {
+	int buferr, reterr;
 
-	    if (errno == EAGAIN) {
-		/* This again was due to O_NONBLOCK, just ignore it. */
-	    } else if (errno == EPIPE) {
+	reterr = buffer_write(cntlr->tcpfd, &td->out_telnet_cmd, &buferr);
+	if (reterr == -1) {
+	    if (buferr == EPIPE) {
 		goto out_fail;
 	    } else {
 		/* Some other bad error. */
 		syslog(LOG_ERR, "The tcp write for controller had error: %m");
 		goto out_fail;
 	    }
-	} else {
-	    int i, j;
-
-	    /* Copy the remaining data. */
-	    for (j=0, i=write_count; i<td->out_telnet_cmd_size; i++, j++)
-		td->out_telnet_cmd[j] = td->out_telnet_cmd[i];
-	    td->out_telnet_cmd_size -= write_count;
-	    if (td->out_telnet_cmd_size != 0)
-		/* If we have more telnet command data to send, don't
-		   send any real data. */
-		goto out;
 	}
+	if (buffer_cursize(&td->out_telnet_cmd) > 0)
+	    /* Still telnet data left, don't send regular data */
+	    goto out;
     }
 
     write_count = write(cntlr->tcpfd,
@@ -628,6 +613,7 @@ handle_accept_port_read(int fd, void *data)
     controller_info_t *cntlr;
     socklen_t         len;
     char              *err = NULL;
+    int               optval;
 
     if (num_controller_ports >= max_controller_ports) {
 	err = "Too many controller ports\n\r";
@@ -671,6 +657,14 @@ handle_accept_port_read(int fd, void *data)
 	goto errout;
     }
 
+    optval = 1;
+    if (setsockopt(cntlr->tcpfd, SOL_SOCKET, SO_KEEPALIVE, (void *)&optval,
+		   sizeof(optval)) == -1) {
+	close(cntlr->tcpfd);
+	syslog(LOG_ERR, "Could not enable SO_KEEPALIVE on the tcp port: %m");
+	goto errout;
+    }
+
     cntlr->inbuf_count = 0;
     cntlr->outbuf = NULL;
     cntlr->monitor_port_id = NULL;
@@ -706,12 +700,12 @@ errout:
 errout2:
     {
 	/* We have a problem so refuse this one. */
-	struct sockaddr_in dummy_sockaddr;
+	struct sockaddr_storage dummy_sockaddr;
 	socklen_t len = sizeof(dummy_sockaddr);
 	int new_fd = accept(fd, (struct sockaddr *) &dummy_sockaddr, &len);
 
 	if (new_fd != -1) {
-	    write(new_fd, err, strlen(err));
+	    write_ignore_fail(new_fd, err, strlen(err));
 	    close(new_fd);
 	}
     }
@@ -721,23 +715,30 @@ errout2:
 int
 controller_init(char *controller_port)
 {
-    struct sockaddr_in sock;
+    struct sockaddr_storage sock;
+    socklen_t sock_len;
     int    optval = 1;
 
-    if (scan_tcp_port(controller_port, &sock) == -1) {
-	return -1;
+    if (scan_tcp_port(controller_port, AF_UNSPEC, &sock, &sock_len) == -1)
+	return CONTROLLER_INVALID_TCP_SPEC;
+    
+    acceptfd = socket(sock.ss_family, SOCK_STREAM, 0);
+    if ((acceptfd == -1) && (errno == EAFNOSUPPORT)) {
+	/* Retry IPV4-only */
+	if (scan_tcp_port(controller_port, AF_INET, &sock, &sock_len) == -1)
+	    return CONTROLLER_INVALID_TCP_SPEC;
+	acceptfd = socket(sock.ss_family, SOCK_STREAM, 0);
     }
-
-    acceptfd = socket(PF_INET, SOCK_STREAM, 0);
     if (acceptfd == -1) {
 	syslog(LOG_ERR, "Unable to create TCP socket: %m");
-	exit(1);
+	return CONTROLLER_CANT_OPEN_PORT;
     }
 
     if (fcntl(acceptfd, F_SETFL, O_NONBLOCK) == -1) {
 	close(acceptfd);
+	acceptfd = -1;
 	syslog(LOG_ERR, "Could not fcntl the accept port: %m");
-	exit(1);
+	return CONTROLLER_CANT_OPEN_PORT;
     }
 
     if (setsockopt(acceptfd,
@@ -746,20 +747,27 @@ controller_init(char *controller_port)
 		   (void *)&optval,
 		   sizeof(optval)) == -1) {
 	close(acceptfd);
+	acceptfd = -1;
 	syslog(LOG_ERR, "Unable to set reuseaddress on socket: %m");
-	exit(1);
+	return CONTROLLER_CANT_OPEN_PORT;
     }
 
-    if (bind(acceptfd, (struct sockaddr *) &sock, sizeof(sock)) == -1) {
+    check_ipv6_only(sock.ss_family,
+		    ((struct sockaddr *) &sock),
+		    acceptfd);
+
+    if (bind(acceptfd, (struct sockaddr *) &sock, sock_len) == -1) {
 	close(acceptfd);
+	acceptfd = -1;
 	syslog(LOG_ERR, "Unable to bind TCP port: %m");
-	exit(1);
+	return CONTROLLER_CANT_OPEN_PORT;
     }
 
     if (listen(acceptfd, 1) != 0) {
 	close(acceptfd);
+	acceptfd = -1;
 	syslog(LOG_ERR, "Unable to listen to TCP port: %m");
-	exit(1);
+	return CONTROLLER_CANT_OPEN_PORT;
     }
 
     sel_set_fd_handlers(ser2net_sel,
@@ -771,3 +779,13 @@ controller_init(char *controller_port)
     sel_set_fd_read_handler(ser2net_sel, acceptfd, SEL_FD_HANDLER_ENABLED);
     return 0;
 }
+
+void
+controller_shutdown(void)
+{
+    if (acceptfd == -1)
+	return;
+    sel_clear_fd_handlers(ser2net_sel, acceptfd);
+    close(acceptfd);
+    acceptfd = -1;
+}
